@@ -29,13 +29,69 @@ from cemento.draw_io.constants import (
     ShapeType,
 )
 from cemento.draw_io.io import get_template_files
-from cemento.draw_io.preprocessing import clean_term, remove_predicate_quotes
+from cemento.draw_io.preprocessing import (
+    clean_term_preserving_quotes,
+    remove_predicate_quotes,
+)
 from cemento.term_matching.constants import RANK_PROPS
 from cemento.term_matching.transforms import substitute_term
-from cemento.utils.utils import aggregate_defaultdict, filter_graph, fst, snd, trd
+from cemento.utils.utils import (
+    filter_graph,
+    fst,
+    get_graph_root_nodes,
+    get_subgraphs,
+    snd,
+    trd,
+)
+
+
+def clean_element_values(
+    elements: dict[str, dict[str, any]],
+) -> dict[str, dict[str, any]]:
+    return {
+        key: (
+            {
+                **element_attr,
+                "value": (
+                    quotes_preserved
+                    if (
+                        quotes_preserved := clean_term_preserving_quotes(
+                            element_attr["value"]
+                        )
+                    )
+                    != '""'
+                    else ""
+                ),
+            }
+            if "value" in element_attr
+            else element_attr
+        )
+        for key, element_attr in elements.items()
+    }
+
+
+def replace_element_value_html_quotes(
+    elements: dict[str, dict[str, any]],
+) -> dict[str, dict[str, any]]:
+    return {
+        key: (
+            {**element_attr, "value": element_attr["value"].replace("&quot;", '"')}
+            if "value" in element_attr
+            else element_attr
+        )
+        for key, element_attr in elements.items()
+    }
 
 
 def parse_elements(file_path: str | Path) -> dict[str, dict[str, any]]:
+    elements = retrieve_elements(file_path)
+    elements = assign_edge_label_attrs(elements)
+    elements = clean_element_values(elements)
+    elements = replace_element_value_html_quotes(elements)
+    return elements
+
+
+def retrieve_elements(file_path: str | Path) -> dict[str, dict[str, any]]:
     # parse elements
     tree = ET.parse(file_path)
 
@@ -48,63 +104,79 @@ def parse_elements(file_path: str | Path) -> dict[str, dict[str, any]]:
         cell_attrs = dict()
         cell_attrs.update(cell.attrib)
 
-        # add nested position attributes if any
-        nested_attrs = dict()
-        for subcell in cell.findall("mxGeometry"):
-            nested_attrs.update(subcell.attrib)
+        nested_attrs = reduce(
+            lambda acc, subcell: acc | subcell.attrib,
+            cell.findall("mxGeometry"),
+            dict(),
+        )
         cell_attrs.update(nested_attrs)
 
-        # add style attributes as data attributes
-        if "style" in cell.attrib:
-            style_terms = dict()
-            style_tags = []
-            for style_attrib_string in cell.attrib["style"].split(";"):
-                style_term = style_attrib_string.split("=")
-                if len(style_term) > 1:  # for styles with values
-                    key, value = style_term
-                    style_terms[key] = value
-                elif style_term[0]:  # for style tags or names
-                    style_tags.append(style_term[0])
-            cell_attrs.update(style_terms)
-            cell_attrs["tags"] = style_tags
-            del cell_attrs["style"]  # remove style to prevent redundancy
-
-        cell_id = cell.attrib["id"]
-        del cell_attrs["id"]  # remove id since it is now used as a key
-        elements[cell_id] = (
-            cell_attrs  # set dictionary of cell key and attribute dictionary
+        style_attrib_str = (
+            cell_attrs["style"].split(";") if "style" in cell_attrs else []
         )
+        style_term_pairs = [tuple(style.split("=")) for style in style_attrib_str]
+        style_terms = reduce(
+            lambda acc, style_pair: acc | {fst(style_pair): snd(style_pair)},
+            filter(lambda style_pair: len(style_pair) > 1, style_term_pairs),
+            dict(),
+        )
+        style_tags = reduce(
+            lambda acc, style: acc + [fst(style)],
+            filter(lambda tag: len(tag) <= 1 and fst(tag), style_term_pairs),
+            list(),
+        )
+        cell_attrs.update(style_terms)
+        cell_attrs["tags"] = style_tags
+        elements[cell_attrs["id"]] = cell_attrs
 
     return elements
 
 
+def assign_edge_label_attrs(
+    elements: dict[str, dict[str, any]],
+) -> dict[str, dict[str, any]]:
+    new_elements = elements
+    to_remove = set()
+    for label_id, data in elements.items():
+        if "tags" in data and "edgeLabel" in data["tags"]:
+            rel_id = data["parent"]
+            new_elements[rel_id]["tags"] += data["tags"]
+            new_elements[rel_id]["value"] = data["value"]
+            to_remove.add(label_id)
+    new_elements = {
+        key: value for key, value in new_elements.items() if key not in to_remove
+    }
+    return new_elements
+
+
 def extract_elements(
     elements: dict[str, dict[str, any]],
-) -> tuple[set[dict[str, any], set[str]]]:
-    # read edges
-    term_ids, rel_ids = set(), set()
-    for element, data in elements.items():
-        # if the vertex attribute is 1 and edgeLabel is not in tags, it is a term (shape)
-        if (
-            "vertex" in data
-            and data["vertex"] == "1"
-            and (
-                "tags" not in data
-                or ("tags" in data and "edgeLabel" not in data["tags"])
-            )
-        ):
-            term_ids.add(element)
-        # if an element has an edgeLabel tag, it is a relationship (connection)
-        elif "tags" in data and "edgeLabel" in data["tags"]:
-            rel_val = data["value"]
-            rel_id = data["parent"]
-            elements[rel_id]["value"] = rel_val
-            rel_ids.add(rel_id)
-        # if an element has value, source and target, it is also a relationship (connection)
-        elif "value" in data and "source" in data and "target" in data:
-            rel_val = data["value"]
-            rel_id = element
-            rel_ids.add(rel_id)
+) -> tuple[set[str], set[str]]:
+    term_ids = set(
+        map(
+            fst,
+            filter(
+                lambda x: "vertex" in (attrs := snd(x))
+                and attrs["vertex"] == "1"
+                and ("tags" not in attrs or "edgeLabel" not in attrs["tags"]),
+                elements.items(),
+            ),
+        )
+    )
+    rel_ids = set(
+        map(
+            fst,
+            filter(
+                lambda x: (
+                    "value" in (attrs := snd(x))
+                    and "source" in attrs
+                    and "target" in attrs
+                )
+                or ("tags" in attrs and "edgeLabel" in attrs["tags"]),
+                elements.items(),
+            ),
+        )
+    )
     # remove root IDs that get added due to draw.io bugs
     reserved_root_ids = {"0", "1"}
     term_ids -= reserved_root_ids
@@ -139,7 +211,7 @@ def generate_graph(
         graph.add_node(
             term_id,
             term_id=term_id,
-            label=clean_term(term),
+            label=term,
             is_literal=('"' in term or "&quot;" in term),
             parent=(
                 term_info["parent"]
@@ -158,7 +230,7 @@ def generate_graph(
                 f"cannot access {e.args[0]} from the attributes of a relationship in the elements dictionary. Please take a look at relationship with id {rel_id}"
             ) from KeyError
 
-        pred = clean_term(elements[rel_id]["value"])
+        pred = elements[rel_id]["value"]
 
         if strat_terms:
             pred, is_strat = substitute_term(pred, strat_terms)
@@ -192,109 +264,64 @@ def add_node_to_digraph(graph: DiGraph, node: tuple[any, dict[str, any]]) -> DiG
     return new_graph
 
 
-# TODO: add property definition to allow parsing container predicates as annotation types
-def parse_containers(
-    graph: DiGraph,
-    strat_terms: set[str] = None,
-    pred_symbol: str = "->",
-    root_id: str = "1",
-) -> DiGraph:
-    new_graph = graph.copy()
-    containers = (
-        data["parent"]
-        for term, data in graph.nodes(data=True)
-        if "parent" in data and data["parent"] is not None
-    )
-    containers = set(filter(lambda x: x != root_id, containers))
-    term_ids = {
-        value: key for key, value in nx.get_node_attributes(graph, "term_id").items()
-    }
-    container_terms = [term_ids[container_id] for container_id in containers]
+# function for generating container lists
+# just store the ids
+def parse_containers(elements: dict[str, dict[str, any]]) -> dict[str, list[str]]:
+    containers = defaultdict(list)
+    for element, attr in elements.items():
+        if "parent" in attr and attr["parent"] not in {"0", "1"}:
+            containers[attr["parent"]].append(element)
+    return containers
 
-    container_child_pairs = (
-        (container_id, term)
-        for term, data in graph.nodes(data=True)
-        if "parent" in data
-        and (container_id := data["parent"]) is not None
-        and container_id in containers
-    )
-    container_children = reduce(
-        aggregate_defaultdict, container_child_pairs, defaultdict(list)
-    )
-    pred_terms, obj_terms = zip(
-        *map(partial(str.split, sep=pred_symbol), container_terms), strict=True
-    )
-    sub_strat_pred_terms, is_strats = list(
-        zip(
-            *map(
-                partial(substitute_term, search_terms=strat_terms, score_cutoff=95),
-                pred_terms,
-            ),
-            strict=True,
-        )
-    )
-    pred_terms = (
-        substitute_term
-        for substitute_term, _ in zip(sub_strat_pred_terms, pred_terms, strict=True)
-    )
-    pred_terms = list(pred_terms)
-    sub_rank_pred_terms, is_ranks = list(
-        zip(
-            *map(
-                partial(substitute_term, search_terms={"rdfs:subClassOf", "rdf:type"}),
-                pred_terms,
-            ),
-            strict=True,
-        )
-    )
-    pred_terms = (
-        substitute_term
-        for substitute_term, _ in zip(sub_rank_pred_terms, pred_terms, strict=True)
-    )
-    new_edge_data = {
-        container_id: {
-            "label": pred_term,
-            "pred_id": f"{container_id}-1",
-            "is_strat": is_strat,
-            "is_rank": is_rank,
-            "is_predicate": True,
-        }
-        for container_id, pred_term, is_strat, is_rank in zip(
-            containers,
-            pred_terms,
-            is_strats,
-            is_ranks,
-            strict=True,
-        )
-    }
-    obj_terms = list(obj_terms)
-    new_edge_objects = {
-        container_id: obj_term
-        for container_id, obj_term in zip(containers, obj_terms, strict=True)
-    }
-    obj_term_data = (
-        {
-            "term_id": f"{container_id}-2",
-            "label": clean_term(obj_term),
-            "is_literal": ('"' in obj_term or "&quot;" in obj_term),
-            "parent": container_id,
-        }
-        for container_id, obj_term in zip(containers, obj_terms, strict=True)
-    )
-    obj_term_nodes = (
-        (obj_term, obj_term_data)
-        for obj_term, obj_term_data in zip(obj_terms, obj_term_data, strict=True)
-    )
-    new_graph.add_nodes_from(obj_term_nodes)
 
-    new_graph_triples = (
-        (child, new_edge_objects[container_id], new_edge_data[container_id])
-        for container_id, children in container_children.items()
-        for child in children
-    )
-    new_graph.add_edges_from(new_graph_triples)
-    new_graph.remove_nodes_from(container_terms)
-    return new_graph
+def get_container_values(
+    containers: dict[str, list[str]], elements: dict[str, dict[str, any]]
+) -> dict[str, str]:
+    return {
+        container_id: (
+            attrs["value"]
+            if (attrs := elements[container_id]) and "value" in attrs
+            else None
+        )
+        for container_id in containers.keys()
+    }
+
+
+def link_container_members(graph: DiGraph, containers: dict[str, list[str]]) -> DiGraph:
+    graph = graph.copy()
+    for container_id, items in containers.items():
+        for item in items:
+            graph.add_edge(container_id, item, label="mds:hasCollectionMember")
+    return graph
+
+
+def get_container_collection_types(
+    graph: DiGraph, container_labels: dict[str, str], containers: dict[str, list[str]]
+) -> dict[str, str]:
+    graph = graph.copy()
+    for container_id in containers.keys():
+        container_type = container_labels[container_id]
+        if not container_type.strip():
+            container_type = "mds:tripleSyntaxSugar"
+        else:
+            # TODO: move search terms to constants file
+            valid_collection_types = {
+                "owl:unionOf",
+                "owl:intersectionOf",
+                "owl:complementOf",
+                "mds:tripleSyntaxSugar",
+            }
+            container_type, did_substitute = substitute_term(
+                container_type,
+                valid_collection_types,
+            )
+            # TODO: move to error check
+            if not did_substitute:
+                raise ValueError(
+                    f"The provided collection header does not seem to match any of the valid collection types. Choose between: {valid_collection_types} or leaving the header blank."
+                )
+        graph.add_edge(container_type, container_id)
+    return graph
 
 
 def relabel_graph_nodes_with_node_attr(
@@ -302,7 +329,11 @@ def relabel_graph_nodes_with_node_attr(
 ) -> DiGraph:
     node_info = nx.get_node_attributes(graph, new_attr_label)
     relabel_mapping = {
-        current_node_label: node_info[current_node_label]
+        current_node_label: (
+            node_info[current_node_label]
+            if current_node_label in node_info
+            else current_node_label
+        )
         for current_node_label in graph.nodes
     }
     return nx.relabel_nodes(graph, relabel_mapping)
@@ -342,15 +373,6 @@ def get_ranked_subgraph(graph: DiGraph) -> DiGraph:
     return filter_graph(graph, lambda data: data["is_strat"])
 
 
-def get_subgraphs(graph: DiGraph) -> list[DiGraph]:
-    subgraphs = nx.weakly_connected_components(graph)
-    return [graph.subgraph(subgraph_nodes).copy() for subgraph_nodes in subgraphs]
-
-
-def get_graph_root_nodes(graph: DiGraph) -> list[any]:
-    return [node for node in graph.nodes if graph.in_degree(node) == 0]
-
-
 def split_multiple_inheritances(
     graph: DiGraph,
 ) -> tuple[list[DiGraph], list[tuple[any, any]]]:
@@ -482,6 +504,8 @@ def get_tree_dividing_line(
     line_start_x, line_end_x = line_start_x + offset_x, line_end_x + offset_x + 1
     _, tree_size_y = get_tree_canvas_size(tree)
     line_y = tree_size_y - line_offset_y + offset_y
+    # TODO: convert this to own function that runs in write_diagram, add offsets to constants for global access to here
+    line_y += 1.5 if no_instance_in_tree(tree) else 0
     line_start_x, _ = translate_coords(line_start_x, 0)
     line_end_x, line_y = translate_coords(line_end_x, line_y)
     return Line(
@@ -507,12 +531,20 @@ def shift_tree(tree: DiGraph, shift_x: float = 0, shift_y: float = 0) -> DiGraph
     return new_tree
 
 
+def no_instance_in_tree(tree: DiGraph) -> bool:
+    return not any([data["is_instance"] for node, data in tree.nodes(data=True)])
+
+
 def conform_tree_positions(trees: list[DiGraph]) -> list[DiGraph]:
     max_y = snd(max(map(get_tree_canvas_size, trees), key=lambda size: snd(size)))
     tree_diffs = (max_y - snd(get_tree_canvas_size(tree)) for tree in trees)
+    no_instance_trees = map(no_instance_in_tree, trees)
     new_trees = [
-        shift_tree(new_tree, shift_y=shift_y)
-        for new_tree, shift_y in zip(trees, tree_diffs, strict=True)
+        # TODO: make scale factor global for access here
+        shift_tree(new_tree, shift_y=(shift_y - (1.5 if no_instance else 0)))
+        for new_tree, shift_y, no_instance in zip(
+            trees, tree_diffs, no_instance_trees, strict=True
+        )
     ]
     return new_trees
 
@@ -788,8 +820,6 @@ def get_rank_strat_connectors(
         )
     )
     inv_shape_id = {value: key for key, value in shape_id.items()}
-    for connector in connectors:
-        print((inv_shape_id[connector.source_id], inv_shape_id[connector.target_id]))
     rank_strat_connectors = [
         connector
         for connector in connectors
